#include <stdlib.h>
#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include <inttypes.h>

#define  F_CPU   16000000
#include <util/delay.h>
#include "uart.h"
#include "usbn2mc.h"
//#include "usbn2mc/fifo.h"
#include "../usbprog_base/firmwarelib/avrupdate.h"



void interrupt_ep_send(void);
void rs232_send(void);
void setup(void);
void transmit(unsigned char);
void transmitBuffer(unsigned char[], int, int);
void transmitBuffer2(unsigned char[], int, int);

volatile int tx1togl=0; 		// inital value of togl bit


int togl3=0;
int togl1=0;


struct {
	char	dwDTERrate[4];   //data terminal rate, in bits per second
	char    bCharFormat;  //num of stop bits (0=1, 1=1.5, 2=2)
	char    bParityType;  //parity (0=none, 1=odd, 2=even, 3=mark, 4=space)
	char    bDataBits;    //data bits (5,6,7,8 or 16)
} usb_cdc_line_coding;


/*  wait function */
void wait_ms(int ms)
{
  	int i;
  	for(i=0;i<ms;i++)
    	_delay_ms(1);
}

enum {
	SEND_ENCAPSULATED_COMMAND = 0,
	GET_ENCAPSULATED_RESPONSE,
	SET_COMM_FEATURE,
	GET_COMM_FEATURE,
	CLEAR_COMM_FEATURE,
	SET_LINE_CODING = 0x20,
	GET_LINE_CODING,
	SET_CONTROL_LINE_STATE,
	SEND_BREAK
};


/* Device Descriptor */

const unsigned char usbrs232[] =
{ 
		0x12,       // 18 length of device descriptor
    	0x01,       // descriptor type = device descriptor
    	0x10,0x01,  // version of usb spec. ( e.g. 1.1)
    	0x02,       // device class
    	0x00,       // device subclass
    	0x00,       // protocol code
    	0x08,       // deep of ep0 fifo in byte (e.g. 8)
    	0x81,0x17,  // vendor id
    	0x64,0x0c,  // product id
    	0x00,0x01,  // revision id (e.g 1.02)
    	0x00,       // index of manuf. string
    	0x00,       // index of product string
    	0x00,       // index of ser. number
    	0x01        // number of configs
};

/* Configuration descriptor */

const unsigned char usbrs232Conf[] =
{ 
	0x09,       // 9 length of this descriptor
    	0x02,       // descriptor type = configuration descriptor
    	0x48,0x00,  // total length with first interface ...
    	0x02,       // number of interfaces //bene 01
    	0x01,       // number if this config. ( arg for setconfig)
    	0x00,       // string index for config
    	0xA0,       // attrib for this configuration ( bus powerded, remote wakup support)
    	0x1A,       // power for this configuration in mA (e.g. 50mA)
		    //InterfaceDescriptor
    	0x09,       // 9 length of this descriptor
    	0x04,       // descriptor type = interface descriptor
    	0x00,       // interface number
    	0x00,       // alternate setting for this interface
    	0x01,       // number endpoints without 0
    	0x02,       // class code
    	0x02,       // sub-class code
    	0x01,       // protocoll code
    	0x00,       // string index for interface

    /* CDC Class-Specific descriptor */
    5,           /* sizeof(usbDescrCDC_HeaderFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    0,           /* header functional descriptor */
    0x10, 0x01,

    4,           /* sizeof(usbDescrCDC_AcmFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    2,           /* abstract control management functional descriptor */
    0x02,        /* SET_LINE_CODING, GET_LINE_CODING, SET_CONTROL_LINE_STATE    */

    5,           /* sizeof(usbDescrCDC_UnionFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    6,           /* union functional descriptor */
    0,           /* CDC_COMM_INTF_ID */
    1,           /* CDC_DATA_INTF_ID */

    5,           /* sizeof(usbDescrCDC_CallMgtFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    1,           /* call management functional descriptor */
    3,           /* allow management on data interface, handles call management by itself */
    1,           /* CDC_DATA_INTF_ID */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    5,           /* descriptor type = endpoint */
    0x83,        /* IN endpoint number 3 */
    0x03,        /* attrib: Interrupt endpoint */
    8, 0,        /* maximum packet size */
    100,         /* in ms */

    /* Interface Descriptor  */
    9,           /* sizeof(usbDescrInterface): length of descriptor in bytes */
    4,           /* descriptor type */
    1,           /* index of this interface */
    0,           /* alternate setting for this interface */
    2,           /* endpoints excl 0: number of endpoint descriptors to follow */
    0x0A,        /* Data Interface Class Codes */
    0,
    0,           /* Data Interface Class Protocol Codes */
    0,           /* string index for interface */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    5,           /* descriptor type = endpoint */
    0x01,        /* OUT endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    8, 0,        /* maximum packet size 8->6 */
    0,           /* in ms */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    5,  /* descriptor type = endpoint */
    0x81,        /* IN endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    8, 0,        /* maximum packet size */
    0,           /* in ms */

};



/* uart interrupt (only for debugging) */

SIGNAL(SIG_UART_RECV)
{
	//UARTWrite("tipp");
	//fifo_put (toUSBFIFO,UARTGetChar());
	//USBNWrite(TXC2,FLUSH);
	//USBNWrite(TXD2,UARTGetChar());
	//rs232_send();	
}

/* interrupt signael from usb controller */

SIGNAL(SIG_INTERRUPT0)
{
	USBNInterrupt();
}


/*************** usb class HID requests  **************/

// reponse for requests on interface
void USBNInterfaceRequests(DeviceRequest *req,EPInfo* ep)
{
}

/* id need for live update of firmware */
void USBNDecodeVendorRequest(DeviceRequest *req)
{
	switch(req->bRequest)
	{
	case STARTAVRUPDATE:
		avrupdate_start();
	break;
	}
}

// class requests
void USBNDecodeClassRequest(DeviceRequest *req,EPInfo* ep)
{
	UARTWrite("class");
	static unsigned char serialNotification[10] = {0xa1,0x20,0,0,0,0,2,0,3,0};
	int loop;
	switch(req->bRequest)
	{
		case 0x20:	//SET_LINE_CODING:
			UARTWrite("set line\r\n");
			USBNWrite(RXC0,RX_EN);
			
			USBNRead(RXD0);
			USBNRead(RXD0);
			USBNRead(RXD0);
			USBNRead(RXD0);
			USBNRead(RXD0);
			USBNRead(RXD0);
			USBNRead(RXD0);
			
			//USBNWrite(RXC0,RX_EN);
			//USBNWrite(RXC0,FLUSH);
		
			USBNWrite(TXC0,FLUSH);
			USBNWrite(TXC0,TX_TOGL+TX_EN);
		break;
		case 0x21:	// GET_LINE_CODING:
			//UARTWrite("get line coding");
			USBNWrite(TXC0,FLUSH);

			// baud rate
			USBNWrite(TXD0,0x80);
			USBNWrite(TXD0,0x25);
			USBNWrite(TXD0,0);
			USBNWrite(TXD0,0);

			USBNWrite(TXD0,0); //stopbit
			USBNWrite(TXD0,0); // parity
			USBNWrite(TXD0,8); // databits

			interrupt_ep_send();

			USBNWrite(TXC0,TX_TOGL+TX_EN);
			
		break;
		case 0x22:	//SET_CONTROL_LINE_STATE:
			//UARTWrite("set ctrl line state");
			USBNWrite(TXC1,FLUSH);
			// fill endpoint fifo
			for(loop=0;loop<8;loop++)
				USBNWrite(TXD1,serialNotification[loop]);

			//send and control togl bit
			interrupt_ep_send();
			USBNWrite(TXC0,TX_TOGL+TX_EN);
		break;
	}
}


// usb zu rs232
void USBtoRS232(char * buf)
{

	//fifo_put(toRS232FIFO,0x33);
	//fifo_put(toRS232FIFO,0x34);
	//fifo_put(toRS232FIFO,0x35);
	//UARTPutChar(buf[0]);

	//USBNWrite(TXC2,FLUSH);
	//USBNWrite(TXD2,0x44);
	//rs232_send();	
	//UARTWrite("usb to rs232");
	
}


// togl pid for in endpoint
void interrupt_ep_send()
{
	if(togl3==1) {
		togl3=0;
		USBNWrite(TXC1,TX_LAST+TX_EN);
	} else {
		togl3=1;
		USBNWrite(TXC1,TX_LAST+TX_EN+TX_TOGL);
	}
}

// togl pid for in endpoint
void rs232_send()
{
	if(togl1==1) {
		togl1=0;
		USBNWrite(TXC2,TX_LAST+TX_EN);
	} else {
		togl1=1;
		USBNWrite(TXC2,TX_LAST+TX_EN+TX_TOGL);
	}
}

void setup() {
	sei();			// activate global interrupts
	//###### Setup USB ###########################
	USBNInit((unsigned char*)usbrs232,(unsigned char*)usbrs232Conf);
	USBNInitMC();		// start usb controller
	USBNStart();		// start device stack
	//####### Setup ADC ##########################
	// Kanal 7
	// LeftAdjust
	// Ref = Vcc
	ADMUX = (0<<REFS1) | (1<<REFS0) | (1<< ADLAR) | (1<<MUX2) | (1<<MUX1) | (1<<MUX0);
	// - div = 128 , @16Mhz
	// --> ADC-Clock = 16Mhz/128 = 125 khz
	// --> SR = ADC-Clock / 13 = 9615,38461538461538461538461538462hz ~ 9,6 khz
	// --> Tiefpass an ADC mit Cutoff bei 3,5khz
	ADCSRA =(1<<ADIF) | (1<<ADEN) | (1<<ADSC) | (1<< ADATE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
	//####### Setup LED
	//Led Port auf Ausgang
	DDRA = (1 << DDA4);

	/*
	unsigned char old = USBNRead(EPC1);
	
	USBNWrite(EPC1, old | (1<<EP_EN));
	unsigned char newv = USBNRead(EPC1);
	while(1) {
		transmit(old);
		transmit(newv);
		transmit('\r');
		transmit('\n');
	}*/

}



/*************** main function  **************/
#define BUFFER_SIZE 500
unsigned char adc_buffer[BUFFER_SIZE];

int main(void) {
	setup();
	int i;
	/*unsigned char text[] = "Hallo Welt\r\n";
	while(1) {
		transmitBuffer(text, 0, 12);
		wait_ms(500);

	}*/


	for(i=0;i<10;i++) {
		PORTA &= ~(1<<PA4); //off
		wait_ms(100);
		PORTA |= (1<<PA4);  //on
		wait_ms(100);
	}
	
	
	while(1) {
		//LED aus
		PORTA &= ~(1<<PA4);
		for(i=0;i<BUFFER_SIZE;i++) {
			//Warten bis ADC fertig
			while (!(ADCSRA & (1<<ADIF))){;}
			//Wegen Left Adjust nur High part lesen!
			adc_buffer[i] = ADCH;
			//Flag lschen
			ADCSRA |= (1<<ADIF) | (1<<ADEN) | (1<<ADSC); 
			if( adc_buffer[i] == 0xFF) {
				/*if (i == BUFFER_SIZE-1) {
					adc_buffer[i]=0xFF-1;
				} else {
					adc_buffer[++i] = 0xFF;
				}*/
				adc_buffer[i]=0xFF-1;
			}
		}
		//LED an
		PORTA |= (1<<PA4);
		
		//Buffer ber Uart ausgeben...
		/*for(i=0;i<BUFFER_SIZE;i++) {
			transmit(adc_buffer[i]);
		}*/
		transmitBuffer2(adc_buffer, 0, BUFFER_SIZE);
		//Ende Zeichen senden
		//transmit((unsigned char)0xFF);
		//transmit((unsigned char)0);
	}
}

void transmitBuffer(unsigned char buffer[], int start, int length) {
	int i;
	for(i=start;i<length;i++) {
		transmit(buffer[i]);
	}
}
#define burst 3
void transmitBuffer2(unsigned char buffer[], int start, int length) {
	int i;
	int j;
	unsigned char status;
	/*USBNWrite(TXC2,FLUSH+TX_EN);
	for(i=start;i<length;i+=burst) {
		for(j=0;j<burst;j++) {
			USBNWrite(TXD2, buffer[i+j]);
		}
		rs232_send();
		interrupt_ep_send();
		wait_ms(20);
		status = USBNRead(TXS2);
		while(!(status & TX_DONE)) {
			status = USBNRead(TXS2);
		}
		transmit(status);
		transmit('\r');
		transmit('\n');
		break;

	}*/
	USBNWrite(TXC2,FLUSH+TX_EN);
	USBNWrite(TXD2, USBNRead(TXS2));
	for(j=0;j<burst;j++) {
		USBNWrite(TXD2, 0xAA);
	}
	USBNWrite(TXD2, USBNRead(TXS2));
	USBNWrite(TXD2, '\r');
	USBNWrite(TXD2, '\n');
	if(togl1==1) {
		togl1=0;
		USBNWrite(TXC2,TX_LAST+TX_EN);
	} else {
		togl1=1;
		USBNWrite(TXC2,TX_LAST+TX_EN+TX_TOGL);
	}
	interrupt_ep_send();
}

void transmit(unsigned char c) {
	USBNWrite(TXC2,FLUSH);
	USBNWrite(TXD2, c);
	rs232_send();
	interrupt_ep_send();
	wait_ms(20);

}






